/*
 * gaim
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * osd.c: On-screen display plugin for Gaim.
 * Copyright (C) 2001 Chris Sperandio <csperandio@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* (cvs $Id: display.c,v 1.19 2004/09/20 06:08:44 deryni9 Exp $) */

#include "gaimosd.h"

/* XOSD handle */
static xosd *Osd = NULL;

static gchar *Osd_safe_font = NULL;

static guint Osd_set_osd_lines_id = 0;
static guint Osd_set_font_id = 0;
static guint Osd_set_h_offset_id = 0;
static guint Osd_set_h_align_id = 0;
static guint Osd_set_v_offset_id = 0;
static guint Osd_set_v_align_id = 0;
static guint Osd_set_shadow_offset_id = 0;

/* XXX fix the whole callback thing to actually use callbacks (instead of
 * calling xosd functions directly from both
 *
 * This is mostly done, only thing left to do is check to see if the xosd bug
 * is still around */
static void osd_display_set_font(gchar *val);

/* OSD_DISPLAY_FREE():
	This function frees osd_display_t memory allocations */
static void
osd_display_free(osd_display_t *disp)
{
	if (!disp)
		return;

	g_source_remove(disp->timeout);
	g_slist_foreach(disp->slist, (GFunc) g_free, NULL);
	g_slist_free(disp->slist);
	g_free(disp);
}

/* OSD_DISPLAY_CB():
	This function is the callback function for the event timer set in
	osd_display().  It scrolls off the current text, while displaying
	any new text.  */
gboolean
osd_display_cb(osd_display_t *disp)
{
	printf("bar 1\n");
	if (!disp)
		return FALSE;

	printf("bar 2\n");
	if (disp->lines > -Osd_lines) {
		disp->lines--;
		XOSD_SCROLL(Osd, 1);
		if (disp->slist) {
			gchar *tmp = (gchar *) disp->slist->data;
			XOSD_DISPLAY(Osd, Osd_lines-1, XOSD_string, tmp);
			disp->slist = g_slist_delete_link(disp->slist, disp->slist);
			g_free(tmp);
		}
		return TRUE;
	}

	printf("bar 3\n");
	XOSD_HIDE(Osd);
	XOSD_SCROLL(Osd, Osd_lines);
	Osd_current = NULL;
	osd_display_free(disp);
	return FALSE;
}

/* OSD_DISPLAY_MSG():
	This function calculates the display message using all of the
	substitutions into one long string. */
static inline gchar *
osd_display_msg(GaimConnection *gc, GaimBuddy *b, gchar *text, gchar *who, gchar *m, gchar *o)
{
	gint i, pos, len;
	gchar *ptr, *new, *replace, curtime[12], *msg;
	gchar *proto, *alias;
	time_t timer = time(0);
	struct tm *tm = localtime(&timer);

	proto = (gc && gc->prpl && gc->prpl->info) ? gc->prpl->info->name : "<protocol>";
	alias = (b && b->alias) ? b->alias : who;
	if (!who) who = alias = "Anonymous";

	ptr = new = msg = g_strdup(text);
	len = strlen(msg)+1;
	while (*ptr != '\0') {

		replace = NULL;

		switch (ptr[0]) {

		/* replace control characters with spaces */
		case '\a':
		case '\b':
		case '\f':
		case '\r':
		case '\v':
			*ptr = ' ';
			break;

		/* replace tab with 8 spaces */
		case '\t':
			replace = "        ";
			break;

		/* string substitutions */
		case '%':

			switch (ptr[1]) {

			case 'p': /* user protocol */
				replace = proto;
				break;

			case 'u': /* user id (same escape as AIM's %u) */
				replace = who;
				break;

			case 'a': /* user alias */
				replace = alias;
				break;

			case 't': /* time (same escape as AIM's %t) */
				strftime(curtime, 12, "%r", tm);
				replace = curtime;
				break;

			case 'T': /* military time (24 hour) */
				strftime(curtime, 12, "%T", tm);
				replace = curtime;
				break;

			case 'd': /* date mm/dd/yyyy */
				strftime(curtime, 12, "%m/%d/%Y", tm);
				replace = curtime;
				break;

			case 'm': /* message */
				replace = (m) ? m : "";
				break;

			case 'o': /* other message (ie, room or level) */
				replace = (o) ? o : "";
				break;

			case '%': /* if the user really wants a %a (etc) in the message */
				replace = "%";
				break;
			}

			break;
		}

		if (replace && (i = strlen(replace)) >= 0) {
			pos = ptr-new; /* position of matching %x replacement */
			len += i-2;
			new = (gchar *) g_malloc(len);
			strncpy(new, msg, pos); /* not terminated with \0 character */
			strcpy(ptr = new+pos, replace);
			strcpy(ptr += i, msg+pos+2); /* ptr is position in new string */
			g_free(msg);
			msg = new;
			ptr--;
		}

		ptr++;
	}
	return msg;
}

/* OSD_DISPLAY_SLIST():
	This function splits up the display message into multiple "screen-sized"
	strings that can be displayed separately. */
static inline GSList *
osd_display_slist(gchar *msg, gint *lines)
{
	gchar *nl, *pos, *end, *ptr;
	gint msg_length;
	GSList *slist = NULL;

	msg_length = gaim_prefs_get_int(GAIMOSD_PREFS "/msg_length");
	if (msg_length <= 0) msg_length = 1;

	*lines = 0;
	ptr = msg;
	while (TRUE) {

		nl = strchr(ptr,'\n');

		if (nl != NULL && nl-ptr <= msg_length) {

			slist = g_slist_append(slist, g_strndup(ptr, nl-ptr));
			ptr = nl + 1;

		} else if (strlen(ptr) > msg_length) {

			end = pos = ptr + msg_length;
			while (pos > ptr && *pos != ' ') pos--;

			if (*pos == ' ') {
				slist = g_slist_append(slist, g_strndup(ptr, pos-ptr));
				ptr = pos + 1;
			} else {
				slist = g_slist_append(slist, g_strndup(ptr, end-ptr));
				ptr = end;
			}
		} else {
			slist = g_slist_append(slist, g_strdup(ptr));
			break;
		}
		(*lines)++;
	}

	return slist;
}

/* OSD_DISPLAY():
	This function is called from any of Gaim's event functions to display
	something. */
void
osd_display(GaimConnection *gc, GaimConversation *c, gchar *who, gint event, gchar *m, gchar *o)
{
	GaimBuddy *b = NULL;
	osd_display_t *disp;
	gchar *tmp, *color, *text;
	gint lines, timeout, i;
	gint flags = GAIMOSD_FLAGS_VOID;
	gint priority = 0;
	GSList *slist;
#if BUILD_OSD_CONVS
	osd_conv_t *oc;
#endif

	if (!Osd)
		return;

#if BUILD_OSD_CONVS
	/* search for conversation */
	if (c && (oc = osd_conv_find(c)) && !oc->display) {
		if (!(event == EVENT_RECEIVED_CHAT_MSG_ME
				&& gaim_prefs_get_bool(GAIMOSD_PREFS "/chat/me_override")))
			return;
	}
#endif

	if (gc) {
#if BUILD_OSD_BUDDIES
		/* search for customized buddy options */
		if ((b = gaim_find_buddy(gc->account, who))) {
			tmp = g_strconcat("gaimosd/", Osd_event[event].name, NULL);
			flags = gaim_blist_node_get_int(&b->node, tmp);
			g_free(tmp);
			priority = gaim_blist_node_get_int(&b->node, "gaimosd/priority");
		}
#endif /* BUILD_OSD_BUDDIES */

		if (flags == GAIMOSD_FLAGS_VOID) {
			tmp = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[event].name, "/flags", NULL);
			flags = gaim_prefs_get_int(tmp) | GAIMOSD_FLAGS_VALID;
			g_free(tmp);
		}

		if (gc->away) {
			if (flags && !(flags & GAIMOSD_FLAGS_SHOW_AWAY))
				return;
		} else {
			if (flags && !(flags & GAIMOSD_FLAGS_SHOW_HERE))
				return;
		}
	}

	tmp = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[event].name, "/priority", NULL);
	priority += gaim_prefs_get_int(tmp);
	g_free(tmp);

	if (Osd_current) {
		if (Osd_current->priority < priority)
			return;
		disp = Osd_current;
		Osd_current = NULL;
		osd_display_free(disp);
	}

	disp = g_new0(osd_display_t,1);
	if (!disp)
		return;

	tmp = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[event].name, "/text", NULL);
	text = (gchar *)gaim_prefs_get_string(tmp);
	g_free(tmp);
	tmp = osd_display_msg(gc, b, text, who, m, o);
	slist = osd_display_slist(tmp, &lines);
	g_free(tmp);
	tmp = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[event].name, "/color", NULL);
	color = (gchar *)gaim_prefs_get_string(tmp);
	g_free(tmp);
	tmp = g_strconcat(GAIMOSD_PREFS "/event/", Osd_event[event].name, "/timeout", NULL);
	timeout = gaim_prefs_get_int(tmp);
	g_free(tmp);

	XOSD_HIDE(Osd);
	XOSD_SCROLL(Osd, Osd_lines);
	XOSD_SET_COLOUR(Osd, color);
	XOSD_SET_TIMEOUT(Osd, -1);

	printf("foo 1\n");
	for (i = 0; i < Osd_lines; i++) {
		if (!slist)
			break;
		tmp = (gchar *) slist->data;
		XOSD_DISPLAY(Osd, i, XOSD_string, tmp);
		slist = g_slist_delete_link(slist, slist);
		g_free(tmp);
		lines--;
	}

	disp->gc = gc;
	disp->c = c;
	disp->priority = priority;
	disp->slist = slist;
	disp->lines = (lines > 0) ? lines : -Osd_lines;

	Osd_current = disp;
	disp->timeout = g_timeout_add(timeout, (GSourceFunc) osd_display_cb, disp);
}

/* OSD_DISPLAY_KILL():
	This function kills/frees all current osd displays (and timers). */
void
osd_display_kill(void)
{
	osd_display_t *disp = Osd_current;

	Osd_current = NULL;
	osd_display_free(disp);

	if (Osd) {
		XOSD_HIDE(Osd);
		XOSD_SCROLL(Osd, Osd_lines);
	}
}

/* OSD_DISPLAY_SET_x():
	These functions are called when prefs are updated via Gaim's pref API. */
static void
osd_display_set_all(void)
{
	/*
	int val;
	*/
	gchar *font= NULL;

	/* XXX try not to have to cast this */
	font = (gchar *)gaim_prefs_get_string(GAIMOSD_PREFS "/xosd/font");
	osd_display_set_font(font);
}

static void
osd_display_set_osd_lines(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	osd_display_kill();
	XOSD_DESTROY(Osd);
	Osd = XOSD_CREATE(GPOINTER_TO_INT(val));
	Osd_lines = GPOINTER_TO_INT(val);

	osd_display_set_all();
}

static void
osd_display_set_font_cb(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	osd_display_set_font(val);
}

static void
osd_display_set_h_offset(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	XOSD_SET_HORIZONTAL_OFFSET(Osd, GPOINTER_TO_INT(val));
}

static void
osd_display_set_h_align(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	XOSD_SET_ALIGN(Osd, GPOINTER_TO_INT(val));
}

static void
osd_display_set_v_offset(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	XOSD_SET_VERTICAL_OFFSET(Osd, GPOINTER_TO_INT(val));
}

static void
osd_display_set_v_align(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	XOSD_SET_POS(Osd, GPOINTER_TO_INT(val));
}

static void
osd_display_set_shadow_offset(const char *name, GaimPrefType type, gpointer val, gpointer data)
{
	XOSD_SET_SHADOW_OFFSET(Osd, GPOINTER_TO_INT(val));
}

static void
osd_display_set_font(gchar *val)
{
	int value ;

	if (!Osd)
		return;

	XOSD_HIDE(Osd);
	if (XOSD_SET_FONT(Osd, val)) {

		gaim_debug_misc("libxosd", "failure loading font %s\n", val);
		gaim_prefs_disconnect_callback(Osd_set_font_id);

		if (XOSD_SET_FONT(Osd, Osd_safe_font)) {
			gaim_debug_misc("libxosd", "failure loading font %s\n", Osd_safe_font);

			XOSD_SET_FONT(Osd, DEF_FONT);
			gaim_prefs_set_string(GAIMOSD_PREFS "/xosd/font", DEF_FONT);
			g_free(Osd_safe_font);
			Osd_safe_font = g_strdup(DEF_FONT);
		} else {
			gaim_prefs_set_string(GAIMOSD_PREFS "/xosd/font", Osd_safe_font);
		}

		Osd_set_font_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/font", &osd_display_set_font_cb, NULL);

	} else {
		g_free(Osd_safe_font);
		Osd_safe_font = g_strdup(val);
	}

	/* xosd bug - need to refresh values */
	value = gaim_prefs_get_int(GAIMOSD_PREFS "/xosd/h_offset");
	XOSD_SET_HORIZONTAL_OFFSET(Osd, value);
	value = gaim_prefs_get_int(GAIMOSD_PREFS "/xosd/h_align");
	XOSD_SET_ALIGN(Osd, value);
	value = gaim_prefs_get_int(GAIMOSD_PREFS "/xosd/v_offset");
	XOSD_SET_VERTICAL_OFFSET(Osd, value);
	value = gaim_prefs_get_int(GAIMOSD_PREFS "/xosd/v_align");
	XOSD_SET_POS(Osd, value);
	value = gaim_prefs_get_int(GAIMOSD_PREFS "/xosd/shadow_offset");
	XOSD_SET_SHADOW_OFFSET(Osd, value);
}

/* OSD_DISPLAY_INIT():
	This function initializes XOSD. */
gboolean
osd_display_init(void)
{
	g_free(Osd_safe_font);
	Osd_safe_font = g_strdup(gaim_prefs_get_string(GAIMOSD_PREFS "/xosd/font"));

	Osd_lines = gaim_prefs_get_int(GAIMOSD_PREFS "/osd_lines");
	Osd = XOSD_CREATE(Osd_lines);
	if (!Osd)
		return FALSE;

	Osd_set_osd_lines_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/osd_lines", &osd_display_set_osd_lines, NULL);
	Osd_set_font_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/font", &osd_display_set_font_cb, NULL);
	Osd_set_h_offset_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/h_offset", &osd_display_set_h_offset, NULL);
	Osd_set_h_align_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/h_align", &osd_display_set_h_align, NULL);
	Osd_set_v_offset_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/v_offset", &osd_display_set_v_offset, NULL);
	Osd_set_v_align_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/v_align", &osd_display_set_v_align, NULL);
	Osd_set_shadow_offset_id = gaim_prefs_connect_callback(GAIMOSD_PREFS "/xosd/shadow_offset", &osd_display_set_shadow_offset, NULL);

	osd_display_set_all();
	return TRUE;
}

/* OSD_DISPLAY_UNINIT():
	This function uninitializes XOSD. */
void
osd_display_uninit(void)
{
	gaim_prefs_disconnect_callback(Osd_set_osd_lines_id);
	gaim_prefs_disconnect_callback(Osd_set_font_id);
	gaim_prefs_disconnect_callback(Osd_set_h_offset_id);
	gaim_prefs_disconnect_callback(Osd_set_h_align_id);
	gaim_prefs_disconnect_callback(Osd_set_v_offset_id);
	gaim_prefs_disconnect_callback(Osd_set_v_align_id);
	gaim_prefs_disconnect_callback(Osd_set_shadow_offset_id);

	g_free(Osd_safe_font);
	Osd_safe_font = NULL;

	osd_display_kill();
	if (Osd) {
		XOSD_DESTROY(Osd);
		Osd = NULL;
	}
}
