/*
 * gaim
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * osd.c: On-screen display plugin for Gaim.
 * Copyright (C) 2001 Chris Sperandio <csperandio@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* (cvs $Id: gaimosd-gtk.c,v 1.22 2004/09/20 06:45:56 deryni9 Exp $) */

/* TODO:
	- fix customized buddies
*/

#include "gaimosd.h"

#include <gtk/gtk.h>

#include "gtkblist.h"
#include "gtkplugin.h"
#include "gtkutils.h"

static GaimPlugin *Plugin = NULL;
static gboolean Dialog_updating = FALSE;
static gulong Dialog_open = 0;

static gint Event_selected = 0;
static GtkTextBuffer *Event_buffer;
static GtkWidget *Event_delay_spin;
static GtkAdjustment *Event_delay_spin_adj;
static GtkWidget *Event_priority_spin;
static GtkAdjustment *Event_priority_spin_adj;
static GtkWidget *Event_show_here_check;
static GtkWidget *Event_show_away_check;
static GtkWidget *Event_text_view;
static GtkWidget *Event_color_dialog;
static GtkWidget *Event_color_label;
static GtkWidget *Event_color_button;

#if BUILD_OSD_BUDDIES
static const gchar *Buddies_selected = NULL;
static gint Buddies_history;
static gint Buddies_count;
static GaimAccount *Buddies_account = NULL;
static GtkWidget *Buddies_opt_menu = NULL;
static GtkWidget *Buddies_remove = NULL;
static GtkWidget *Buddies_entry = NULL;
static GtkWidget *Buddies_radios[N_EVENTS*3];
static GtkWidget *Buddies_priority_spin = NULL;
static GtkAdjustment *Buddies_priority_spin_adj = NULL;

static gint osd_buddy_get(GaimBuddy *buddy, const gchar *conf);
static void osd_buddy_set(GaimBuddy *buddy, const gchar *conf, gint val);
static void osd_buddy_add(GaimBuddy *buddy);
static void osd_buddies_update_ui(GaimBuddy *buddy);
static GtkWidget *osd_buddies_menu(GaimBuddy *select, gboolean none);
static void osd_buddies_remove(GtkWidget *button, GaimBuddy *buddy);
#endif

/* COLOR DIALOG ---------------------------------------------------- */

/* partially stolen from Gaim's prefs.c - used to set color of widgets */
static GtkWidget *
osd_color_prefs(GtkWidget *widget, gint *color)
{
	GdkColor c;
	GtkStyle *style;

	c.pixel = 0;
	c.red   = color[0] << 8;
	c.green = color[1] << 8;
	c.blue  = color[2] << 8;

	/* XXX don't work
	gtk_widget_modify_bg(widget, GTK_STATE_NORMAL, &c);
	gtk_widget_modify_bg(widget, GTK_STATE_ACTIVE, &c);
	*/
	style = gtk_style_new ();
	style->bg[0] = c;
	style->bg[1] = c;

	gtk_widget_set_style (widget, style);
	gtk_style_unref(style);
	return widget;
}

static void
osd_color_close(GtkButton *button, gpointer p)
{
	GtkWidget *window;

	g_return_if_fail(Event_color_dialog != NULL);

	window = Event_color_dialog;
	Event_color_dialog = NULL;
	gtk_widget_destroy(window);
}

static void
osd_color_save(GtkButton *button, gpointer p)
{
	GtkWidget *colorsel;
	gdouble c[4];
	gint color[3];
	gint i;
	gchar new[8];
	gchar *tmp;

	g_return_if_fail(Event_color_dialog != NULL);

	colorsel = GTK_COLOR_SELECTION_DIALOG(Event_color_dialog)->colorsel;
	gtk_color_selection_get_color(GTK_COLOR_SELECTION(colorsel), c);

	for (i = 0; i < 3; i++)
		color[i] = c[i] * 255.0;

	g_snprintf(new, 8, "#%02x%02x%02x", color[0], color[1], color[2]);
	tmp = g_strconcat(GAIMOSD_PREFS "/event/",
	                  Osd_event[Event_selected].name, "/color", NULL);
	gaim_prefs_set_string(tmp, new);
	g_free(tmp);

	/* update dialog labels, colors, etc. */
	osd_color_prefs (Event_color_button, color);
	gtk_label_set_text (GTK_LABEL(Event_color_label), new);

	osd_color_close(NULL, NULL);
}

static void
osd_color_dialog(GtkButton *button, gpointer p)
{
	GtkWidget *window;
	GtkWidget *colorsel;
	gdouble c[4];
	gint i, color[3];
	gchar *tmp;

	g_return_if_fail(Event_color_dialog == NULL);

	Event_color_dialog = window = gtk_color_selection_dialog_new (_("Select Font Color"));

	tmp = g_strconcat(GAIMOSD_PREFS "/event/",
	                  Osd_event[Event_selected].name, "/color", NULL);
	sscanf(gaim_prefs_get_string(tmp), "#%02x%02x%02x", &color[0],
	       &color[1], &color[2]);
	g_free(tmp);
	for (i = 0; i < 3; i++)
		c[i] = (gdouble) color[i]/255.0;
	c[3] = 1.0;

	colorsel = GTK_COLOR_SELECTION_DIALOG(window)->colorsel;
	gtk_color_selection_set_color(GTK_COLOR_SELECTION(colorsel), c);
	gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG(window)->help_button);
	g_signal_connect (G_OBJECT(GTK_COLOR_SELECTION_DIALOG(window)->cancel_button),
	                  "clicked", G_CALLBACK(osd_color_close), NULL);
	g_signal_connect (G_OBJECT(GTK_COLOR_SELECTION_DIALOG(window)->ok_button),
	                  "clicked", G_CALLBACK(osd_color_save), NULL);
	g_signal_connect (G_OBJECT(window), "destroy",
	                  G_CALLBACK(osd_color_close), NULL);
	gtk_widget_show_all (window);
}


/* OSD BUDDIES CONFIGURATION --------------------------------------- */

#if BUILD_OSD_BUDDIES
static gint
osd_buddy_get(GaimBuddy *buddy, const gchar *conf)
{
	gchar *tmp;
	gint val;

	if (!buddy)
		return 0;

	tmp = g_strconcat("gaimosd/", conf, NULL);
	val = gaim_blist_node_get_int(&buddy->node, tmp);
	g_free(tmp);
	return val;
}

static void
osd_buddy_set(GaimBuddy *buddy, const gchar *conf, gint val)
{
	gchar *tmp;

	if (!buddy)
		return;

	// don't cause an infinite loop with add!
	if (strcmp("customized", conf) && !osd_buddy_get(buddy, "customized")) {
		osd_buddy_add(buddy);
	}

	tmp = g_strconcat("gaimosd/", conf, NULL);
	gaim_blist_node_set_int(&buddy->node, tmp, val);
	g_free(tmp);

	osd_buddies_update_ui(buddy);
}

static void
osd_buddy_add(GaimBuddy *buddy)
{
	int i;

	osd_buddy_set(buddy, "customized", TRUE);
	for (i = 0; i < N_EVENTS; i++) {
		if (!osd_buddy_get(buddy, Osd_event[i].name)) {
			osd_buddy_set(buddy, Osd_event[i].name, GAIMOSD_FLAGS_VALID);
		}
	}
	if (!osd_buddy_get(buddy, "priority")) {
		osd_buddy_set(buddy, "priority", DEF_BUDDY_PRIORITY);
	}
	osd_buddies_menu(buddy, FALSE);
}

static void
osd_buddy_remove_setting(GaimBuddy *buddy, const gchar *conf)
{
	gchar *tmp;

	if (!buddy)
		return;

	tmp = g_strconcat("gaimosd/", conf, NULL);
	gaim_blist_node_remove_setting(&buddy->node, tmp);
	g_free(tmp);

}



static void
osd_buddies_update_ui(GaimBuddy *buddy)
{
	int i;

	if (Dialog_updating || !Dialog_open)
		return;

	Dialog_updating = TRUE;

	if (!Buddies_account || !buddy) {

		Buddies_selected = NULL;
		gtk_widget_set_sensitive(Buddies_remove, FALSE);
		gtk_adjustment_set_value(Buddies_priority_spin_adj,
		                         DEF_BUDDY_PRIORITY);
		for (i = 0; i < N_EVENTS; i++) {
			gtk_widget_set_sensitive (Buddies_radios[i*3], FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Buddies_radios[i*3]), TRUE);
			gtk_widget_set_sensitive (Buddies_radios[i*3+1], FALSE);
			gtk_widget_set_sensitive (Buddies_radios[i*3+2], FALSE);
		}
		gtk_widget_set_sensitive (Buddies_priority_spin, FALSE);

	} else if (buddy) {

		Buddies_selected = buddy->name;
		gtk_widget_set_sensitive(Buddies_remove, TRUE);

		/* buddy found, allow user to section options */
		for (i = 0; i < N_EVENTS*3; i++)
			gtk_widget_set_sensitive (Buddies_radios[i], TRUE);
		gtk_widget_set_sensitive(Buddies_priority_spin, TRUE);

		/* set buddy's name in text entry */
		gtk_entry_set_text(GTK_ENTRY(Buddies_entry), g_strdup(buddy->name));

		if (osd_buddy_get(buddy, "customized")) {

			/* set buddy-specific options in UI */
			for (i = 0; i < N_EVENTS; i++) {
				switch (osd_buddy_get(buddy, Osd_event[i].name)) {
				case GAIMOSD_FLAGS_SHOW_ALWAYS:
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Buddies_radios[i*3+1]), TRUE);
					break;
				case GAIMOSD_FLAGS_SHOW_NEVER:
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Buddies_radios[i*3+2]), TRUE);
					break;
				default:
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Buddies_radios[i*3]), TRUE);
				}
			}
			gtk_adjustment_set_value(Buddies_priority_spin_adj,
			                         osd_buddy_get(buddy, "priority"));
		} else {
			for (i = 0; i < N_EVENTS; i++) {
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(Buddies_radios[i*3]), TRUE);
			}
		}
	}

	Dialog_updating = FALSE;
}



static void
osd_buddies_select(GtkWidget *widget, GaimBuddy *buddy)
{
	osd_buddies_update_ui(buddy);
}

static void
osd_buddies_menu_build(gpointer key, gpointer value, gpointer user_data)
{
	GtkWidget *item = NULL, *menu = NULL;
	GaimBuddy *buddy = NULL;

	buddy = (GaimBuddy *)value;
	menu = (GtkWidget *)user_data;

	if (buddy && buddy->account == Buddies_account &&
	    osd_buddy_get(buddy, "customized")) {

		if (!Buddies_selected || Buddies_selected == buddy->name) {
			Buddies_selected = buddy->name;
			Buddies_history = Buddies_count;
		}
		item = gtk_menu_item_new_with_label (buddy->name);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		g_signal_connect(G_OBJECT(item), "activate",
		                 G_CALLBACK(osd_buddies_select), buddy);
		Buddies_count++;
	}
}

static GtkWidget *
osd_buddies_menu(GaimBuddy *select, gboolean none)
{
	GtkWidget *menu = NULL, *menuitem = NULL;
	GtkWidget *opt_menu = Buddies_opt_menu;
	GaimBuddyList *blist;

	Buddies_history = -1;
	Buddies_count = 0;
	Buddies_selected = (select) ? select->name : NULL;

	if ((gtk_option_menu_get_menu (GTK_OPTION_MENU (opt_menu)))) {
		gtk_option_menu_remove_menu (GTK_OPTION_MENU (opt_menu));
	}
	menu = gtk_menu_new ();

	blist = gaim_get_blist();
	if (blist)
		g_hash_table_foreach(blist->buddies, osd_buddies_menu_build, menu);

	menuitem = gtk_menu_item_new_with_label(_("No buddy selected"));
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
	gtk_widget_set_sensitive(menuitem, FALSE);

	if (Buddies_history == -1 || none) {
		Buddies_selected = NULL;
		Buddies_history = Buddies_count;
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (opt_menu), Buddies_history);
	gtk_widget_show_all (menu);

	return menu;
}

static void
osd_buddies_remove(GtkWidget *button, GaimBuddy *buddy)
{
	gchar *who;
	gint i;

	if (!Buddies_account)
		return;


	if (!buddy) {
		who = gtk_editable_get_chars (GTK_EDITABLE(Buddies_entry), 0, -1);
		buddy = gaim_find_buddy(Buddies_account, who);
		g_free(who);
		if (!buddy) return;
	}

	Dialog_updating = TRUE;

	osd_buddy_remove_setting(buddy, "customized");
	for (i = 0; i < N_EVENTS; i++) {
		osd_buddy_remove_setting(buddy, Osd_event[i].name);
	}
	osd_buddy_remove_setting(buddy, "priority");

	if (Buddies_selected == buddy->name) {
		gtk_entry_set_text(GTK_ENTRY(Buddies_entry), "");
	}

	Dialog_updating = FALSE;

	osd_buddies_menu(NULL, FALSE);
	osd_buddies_update_ui(gaim_find_buddy(Buddies_account,Buddies_selected));
}

static void
osd_buddies_search(GtkEntry *entry, gpointer p)
{
	GaimBuddy *buddy;
	gchar *who;

	if (Dialog_updating)
		return;

	who = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
	buddy = (Buddies_account) ? gaim_find_buddy(Buddies_account, who) : NULL;
	g_free(who);

	osd_buddies_menu(buddy, (buddy) ? FALSE : TRUE);
	osd_buddies_select(NULL, buddy);
}

static void
osd_buddies_select_account(GtkWidget *opt, GaimAccount *account)
{
	Buddies_account = account;
	gtk_entry_set_text(GTK_ENTRY(Buddies_entry), "");
	osd_buddies_menu(NULL, FALSE);
	osd_buddies_update_ui(gaim_find_buddy(Buddies_account,Buddies_selected));
}


/* BUDDY DRAW MENU ------------------------------------------------- */


static void
osd_draw_buddy_menu_on(GtkWidget *w, GaimBuddy *buddy)
{
	osd_buddy_set(buddy, g_object_get_data(G_OBJECT(w), "event"),
	              GAIMOSD_FLAGS_SHOW_ALWAYS);
	osd_buddies_update_ui(buddy);
}

static void
osd_draw_buddy_menu_off(GtkWidget *w, GaimBuddy *buddy)
{
	osd_buddy_set(buddy, g_object_get_data(G_OBJECT(w), "event"),
	              GAIMOSD_FLAGS_SHOW_NEVER);
	osd_buddies_update_ui(buddy);
}

static void
osd_draw_buddy_menu_def(GtkWidget *w, GaimBuddy *buddy)
{
	osd_buddy_set(buddy, g_object_get_data(G_OBJECT(w), "event"),
	              GAIMOSD_FLAGS_VALID);
	osd_buddies_update_ui(buddy);
}

/* XXX Need to update this so that it conforms to the new (and annoying)
 * blist-node-extended-menu signal */
static gboolean
osd_draw_buddy_menu(GtkMenu *menu, GaimBuddy *buddy)
{
	GtkWidget *item = NULL, *submenu = NULL, *submenu2 = NULL;
	GtkWidget *item_def, *item_on, *item_off;
	gint event;

	/* event submenu */
	submenu = gtk_menu_new ();

	item = gtk_menu_item_new_with_label(buddy->name);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);

	gaim_separator(submenu);

	for (event = 0; event < N_EVENTS; event++) {
		/* option submenu */
		submenu2 = gtk_menu_new();
		GSList *group = NULL;

		item_def = gtk_radio_menu_item_new_with_label(group, _("Use default"));
		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item_def));
		gtk_menu_shell_append(GTK_MENU_SHELL(submenu2), item_def);

		item_on = gtk_radio_menu_item_new_with_label(group, _("Always display"));
		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item_on));
		gtk_menu_shell_append(GTK_MENU_SHELL(submenu2), item_on);

		item_off = gtk_radio_menu_item_new_with_label(group, _("Never display"));
		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item_off));
		gtk_menu_shell_append(GTK_MENU_SHELL(submenu2), item_off);

		switch (osd_buddy_get(buddy, Osd_event[event].name)) {
		case GAIMOSD_FLAGS_SHOW_ALWAYS:
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item_on), TRUE);
			break;
		case GAIMOSD_FLAGS_SHOW_NEVER:
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item_off), TRUE);
			break;
		default:
			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item_def), TRUE);
		}
		g_object_set_data(G_OBJECT(item_on), "event",
		                  (gpointer*)Osd_event[event].name);
		g_object_set_data(G_OBJECT(item_off), "event",
		                  (gpointer*)Osd_event[event].name);
		g_object_set_data(G_OBJECT(item_def), "event",
		                  (gpointer*)Osd_event[event].name);

		g_signal_connect(G_OBJECT(item_on), "toggled",
		                 G_CALLBACK (osd_draw_buddy_menu_on), buddy);
		g_signal_connect(G_OBJECT(item_off), "toggled",
		                 G_CALLBACK (osd_draw_buddy_menu_off), buddy);
		g_signal_connect(G_OBJECT(item_def), "toggled",
		                 G_CALLBACK (osd_draw_buddy_menu_def), buddy);

		/* append option submenu to event submenu */
		item = gtk_menu_item_new_with_label (Osd_event[event].desc);
		gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), submenu2);
		gtk_menu_shell_append (GTK_MENU_SHELL(submenu), item);
	}
	gaim_separator(submenu);

	item = gtk_check_menu_item_new_with_label(_("Use default settings"));
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), !osd_buddy_get(buddy, "customized"));
	g_signal_connect(G_OBJECT(item), "toggled",
	                 G_CALLBACK (osd_buddies_remove), buddy);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item);

	item = gtk_menu_item_new_with_label("GaimOSD");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	gtk_widget_show_all(submenu);

	return TRUE;
}

#endif /* BUILD_OSD_BUDDIES */

/* CONFIG DIALOG - WIDGETS & CALLBACK FUNCTIONS -------------------- */
static void osd_config_event_set_sensitive(gint flags)
{
	flags &= GAIMOSD_FLAGS_SHOW_HERE | GAIMOSD_FLAGS_SHOW_AWAY;
	gtk_widget_set_sensitive (Event_color_button, flags);
	gtk_widget_set_sensitive (Event_delay_spin, flags);
	gtk_widget_set_sensitive (Event_priority_spin, flags);
	gtk_widget_set_sensitive (Event_text_view, flags);
}

static void
osd_config_select_event(GtkEditable *edit, gpointer p)
{
	gint color[3];
	gchar *tmp, *node, *c;
	gint flags;

	Event_selected = GPOINTER_TO_INT (p);
	Dialog_updating = TRUE;

	node = g_strconcat(GAIMOSD_PREFS "/event/",
	                   Osd_event[Event_selected].name, NULL);

	tmp =  g_strconcat(node, "/text", NULL);
	gtk_text_buffer_set_text (Event_buffer, gaim_prefs_get_string(tmp), -1);
	g_free(tmp);
	tmp =  g_strconcat(node, "/timeout", NULL);
	gtk_adjustment_set_value (Event_delay_spin_adj, gaim_prefs_get_int(tmp));
	g_free(tmp);
	tmp =  g_strconcat(node, "/priority", NULL);
	gtk_adjustment_set_value (Event_priority_spin_adj, gaim_prefs_get_int(tmp));
	g_free(tmp);

	tmp =  g_strconcat(node, "/color", NULL);
	c = (gchar *)gaim_prefs_get_string(tmp);
	g_free(tmp);
	sscanf(c, "#%02x%02x%02x", &color[0], &color[1], &color[2]);
	osd_color_prefs (Event_color_button, color);
	gtk_label_set_text (GTK_LABEL(Event_color_label), c);

	tmp = g_strconcat(node, "/flags", NULL);
	flags = gaim_prefs_get_int(tmp);
	g_free(tmp);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (Event_show_here_check),
	                              flags & GAIMOSD_FLAGS_SHOW_HERE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (Event_show_away_check),
	                              flags & GAIMOSD_FLAGS_SHOW_AWAY);

	osd_config_event_set_sensitive(flags);
	g_free(node);
	Dialog_updating = FALSE;
}

static void
osd_open_url(GtkButton *button, gpointer p)
{
	gaim_notify_uri(Plugin, (gchar *)p);
}

static void
osd_set_spin(GtkSpinButton *spin, gpointer p)
{
	gchar *tmp;
	gchar *name = p;
	gint data;

	if (Dialog_updating)
		return;

	data = gtk_spin_button_get_value_as_int(spin);

#if BUILD_OSD_BUDDIES
	if (name[0] != '/') {
		GaimBuddy *buddy = gaim_find_buddy(Buddies_account,
		                                   Buddies_selected);
		osd_buddy_set(buddy, name, data);
	} else
#endif
	{
		tmp = gaim_strreplace(name, "@EVENT@",
		                      Osd_event[Event_selected].name);
		gaim_prefs_set_int(tmp, data);
		g_free(tmp);
	}
}

static GtkWidget *
osd_config_spin(GtkWidget *box, GtkSizeGroup *sg, const gchar *name,
                const gchar *desc, gdouble lower, gdouble upper,
                gdouble step_inc, gdouble page_inc, gdouble page_size)
{
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
	GtkWidget *label = gtk_label_new (desc);
	GtkObject *spin_adj;
	GtkWidget *spin;
	gchar *tmp;

#if BUILD_OSD_BUDDIES
	if (name[0] != '/') {
		spin_adj = gtk_adjustment_new(DEF_BUDDY_PRIORITY, lower, upper,
		                              step_inc, page_inc, page_size);
	} else
#endif
	{
		tmp = gaim_strreplace(name, "@EVENT@",
		                      Osd_event[Event_selected].name);
		spin_adj = gtk_adjustment_new(gaim_prefs_get_int(tmp),
		                              lower, upper, step_inc,
		                              page_inc, page_size);
		g_free(tmp);
	}
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(spin_adj), 1, 0);

	gtk_size_group_add_widget(sg, label);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_widget_set_size_request(spin, 65, -1);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 12);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);

	/* XXX try not to have to cast this */
	g_signal_connect(G_OBJECT (spin), "value-changed",
	                 G_CALLBACK(osd_set_spin), (gchar *)name);
	gtk_widget_show_all (hbox);

	return spin;
}

static void
osd_set_check(GtkToggleButton *toggle, gpointer p)
{
	gchar *tmp, *name;
	gint flag, val;

	if (Dialog_updating)
		return;

	name = gaim_strreplace(p,"@EVENT@",Osd_event[Event_selected].name);
	if ((tmp = g_object_get_data(G_OBJECT(toggle),"flag"))) {
		flag = GPOINTER_TO_INT(tmp);
		val = gaim_prefs_get_int(name) & ~flag;
		if (gtk_toggle_button_get_active(toggle))
			val |= flag;
		gaim_prefs_set_int(name, val);
		osd_config_event_set_sensitive(val);
	} else {
		gaim_prefs_set_bool(name, gtk_toggle_button_get_active(toggle));
	}

	g_free(name);
}
static GtkWidget *
osd_config_check(GtkWidget *box, const gchar *name, const gchar *desc, gint flag)
{
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
	GtkWidget *check = gtk_check_button_new_with_label (desc);
	gchar *tmp;

	gtk_box_pack_start (GTK_BOX (hbox), check, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);
	tmp = gaim_strreplace(name,"@EVENT@",Osd_event[Event_selected].name);
	if (flag > 0) {
		g_object_set_data(G_OBJECT(check),"flag",GINT_TO_POINTER(flag));
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
		                             gaim_prefs_get_int(tmp));
	} else {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
		                             gaim_prefs_get_bool(tmp));
	}
	g_free(tmp);
	g_signal_connect (G_OBJECT (check), "toggled",
	                  G_CALLBACK (osd_set_check), (void*)name);
	gtk_widget_show (check);

	return check;
}

static void
osd_set_radio(GtkToggleButton *toggle, gpointer p)
{
	gchar *tmp;
	gchar *name = p;
	gint data;

	if (Dialog_updating)
		return;

	if (gtk_toggle_button_get_active(toggle)) {
		data = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(toggle),"index"));
#if BUILD_OSD_BUDDIES
		if (name[0] != '/') {
			GaimBuddy *buddy = gaim_find_buddy(Buddies_account, Buddies_selected);
			osd_buddy_set(buddy, name, data);
		} else
#endif
		{
			tmp = gaim_strreplace(name, "@EVENT@",
			                      Osd_event[Event_selected].name);
			gaim_prefs_set_int(tmp, data);
			g_free(tmp);
		}
	}
}

static GtkWidget *
osd_config_radio(GtkWidget *box, GtkSizeGroup *sg, const gchar *name,
                 const gchar *desc, gint num, ...)
{
	va_list radios;
	gchar *tmp;
	GtkWidget *label = gtk_label_new (desc);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
	GtkWidget *hbox1 = gtk_hbox_new (TRUE, 0);
	GtkWidget *radio[num];
	GtkWidget **export;
	gint config_idx[num];
	gint i;
	gint selected = 0;

	va_start(radios, num);
	for (i=0; i < num; i++) {
		tmp = va_arg(radios, gchar *);
		config_idx[i] = va_arg(radios, gint);
		export = va_arg(radios, GtkWidget **);

		if (i == 0)
			radio[0] = gtk_radio_button_new_with_label (NULL, tmp);
		else
			radio[i] = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON(radio[i-1]), tmp);

		if (export)
			*export = radio[i];
		gtk_box_pack_start (GTK_BOX (hbox1), radio[i], TRUE, TRUE, 0);
		g_object_set_data(G_OBJECT(radio[i]),"index",
		                  GINT_TO_POINTER(config_idx[i]));

		/* XXX try to not have to cast this */
		g_signal_connect(G_OBJECT(radio[i]), "toggled",
		                 G_CALLBACK (osd_set_radio), (gchar *)name);
	}
	va_end(radios);

	if (name[0] == '/') {
		tmp = gaim_strreplace(name, "@EVENT@",
		                      Osd_event[Event_selected].name);
		selected = config_idx[gaim_prefs_get_int(tmp)];
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio[selected]), TRUE);
		g_free(tmp);
	}

	gtk_size_group_add_widget (sg, label);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_set_spacing (GTK_BOX (hbox1), 10);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), hbox1, FALSE, FALSE, 10);
	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

	gtk_widget_show_all (hbox);

	return hbox1;
}

static void
osd_set_entry(GtkEditable *edit, gpointer p)
{
	gchar *text;
	gchar *tmp;

	if (Dialog_updating)
		return;

	tmp = gaim_strreplace(p,"@EVENT@",Osd_event[Event_selected].name);
	text = gtk_editable_get_chars (edit, 0, -1);
	gaim_prefs_set_string(tmp, text);
	g_free(text);
	g_free(tmp);
}

static GtkWidget *
osd_config_entry(GtkWidget *box, GtkSizeGroup *sg, gchar *signal,
                 const gchar *name, const gchar *desc)
{
	GtkWidget *label = gtk_label_new (desc);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 11);
	GtkWidget *entry =  gtk_entry_new ();
	gchar *tmp;

	tmp = gaim_strreplace(name,"@EVENT@",Osd_event[Event_selected].name);
	gtk_entry_set_text (GTK_ENTRY (entry),
	                    g_strdup(gaim_prefs_get_string(tmp)));
	g_free(tmp);

	gtk_size_group_add_widget (sg, label);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);
	gtk_widget_show_all (hbox);

	g_signal_connect (G_OBJECT (entry), signal,
	                  G_CALLBACK (osd_set_entry), (void*)name);
	return entry;
}

static void
osd_set_buffer(GtkTextBuffer *buffer, gpointer p)
{
	GtkTextIter start, end;
	gchar *text, *tmp;

	if (Dialog_updating)
		return;

	gtk_text_buffer_get_bounds (buffer, &start, &end);
	text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	tmp = gaim_strreplace(p,"@EVENT@",Osd_event[Event_selected].name);
	gaim_prefs_set_string(tmp, text);
	g_free(tmp);
	g_free(text);
}

static GtkTextBuffer *
osd_config_buffer(GtkWidget *box, const gchar *name, GtkWidget **text_view)
{
	GtkWidget *scrolled = gtk_scrolled_window_new (NULL, NULL);
	GtkWidget *text = gtk_text_view_new ();
	GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(text));
	gchar *tmp;

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
	                                GTK_POLICY_AUTOMATIC,
	                                GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
	                                     GTK_SHADOW_IN);
	tmp = gaim_strreplace(name, "@EVENT@", Osd_event[Event_selected].name);
	gtk_text_buffer_set_text (buffer, gaim_prefs_get_string(tmp), -1);
	g_free(tmp);

	gtk_container_add (GTK_CONTAINER (scrolled), text);
	gtk_box_pack_start (GTK_BOX (box), scrolled, TRUE, TRUE, 0);
	gtk_widget_show_all (scrolled);

	g_signal_connect (G_OBJECT (buffer), "changed",
	                  G_CALLBACK (osd_set_buffer), (void*)name);

	*text_view = text;
	return buffer;
}

static GtkWidget *
osd_config_button(GtkWidget *box, GtkSizeGroup *sg, GtkWidget *button,
                  gchar *desc, gboolean fill, GCallback cb, gpointer p)
{
	GtkWidget *label = gtk_label_new (desc);
	GtkWidget *hbox = gtk_hbox_new (FALSE, 11);

	gtk_size_group_add_widget (sg, label);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), button, fill, fill, 0);
	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);
	gtk_widget_show_all (hbox);

	g_signal_connect (G_OBJECT (button), "clicked", cb, p);

	return button;
}

static GtkWidget *
osd_config_macro(GtkWidget *box, GtkSizeGroup *sg, gchar *macro, gchar *desc)
{
	GtkWidget *hbox = gtk_hbox_new (FALSE, 11);
	GtkWidget *label = gtk_label_new (NULL);
	gchar labeltext[256];

	g_snprintf(labeltext, sizeof(labeltext),
	           "<span weight=\"bold\">%s</span>", macro);
	gtk_label_set_markup (GTK_LABEL (label), labeltext);
	gtk_size_group_add_widget (sg, label);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	label = gtk_label_new (desc);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

	return label;
}

/* CONFIG DIALOG --------------------------------------------------- */
static GtkWidget *
osd_config_page_display(void)
{
	GtkWidget *page;
	GtkWidget *vbox;
	GtkWidget *widget;
	GtkSizeGroup *sg;
	GtkSizeGroup *sg2;

	page = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	vbox = gaim_gtk_make_frame(page, _("Font"));
	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	osd_config_entry(vbox, sg, "activate", GAIMOSD_PREFS "/xosd/font",
	                 _("Font XLFD"));
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/xosd/shadow_offset",
	                _("Shadow Offset"), 0, 100, 1, 10, 10);

	vbox = gaim_gtk_make_frame(page, _("Position"));
	sg2 = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	widget = osd_config_radio(vbox, sg, GAIMOSD_PREFS "/xosd/h_align",
	                          _("Horizontal Alignment"), 3,
	                          _("Left"), XOSD_left, NULL,
	                          _("Center"), XOSD_center, NULL,
	                          _("Right"), XOSD_right, NULL);
	gtk_size_group_add_widget (sg2, widget);
	widget = osd_config_radio(vbox, sg, GAIMOSD_PREFS "/xosd/v_align",
	                          _("Vertical Alignment"), 3,
	                          _("Top"), XOSD_top, NULL,
	                          _("Middle"), XOSD_middle, NULL,
	                          _("Bottom"), XOSD_bottom, NULL);
	gtk_size_group_add_widget (sg2, widget);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/xosd/h_offset",
	                _("Horizontal Offset"), -1000, 1000, 1, 10, 10);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/xosd/v_offset",
	                _("Vertical Offset"), -1000, 1000, 1, 10, 10);

	vbox = gaim_gtk_make_frame(page, _("Display"));
	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/osd_lines",
	                _("Maximum number of OSD lines"), 1,
	                OSD_MAX_LINES, 1, 1, 1);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/msg_length",
	                _("Maximum characters per line"), 0, 1000, 1, 10, 10);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/signon_timeout",
	                _("Signon no-display timeout (msec)"), 0,
	                60000, 250, 1000, 1000);
	osd_config_spin(vbox, sg, GAIMOSD_PREFS "/signon_priority",
	                _("Signon no-display priority"), 0, 100, 1, 10, 10);
	return page;
}

static GtkWidget *
osd_config_page_windows(void)
{
	GtkWidget *page;
	GtkWidget *vbox;

	page = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	vbox = gaim_gtk_make_frame(page, _("IM Conversations"));
#if BUILD_OSD_CONVS
	osd_config_check(vbox, GAIMOSD_PREFS "/conv/display_def",
	                 _("Default with OSD Display On"), 0);
	osd_config_check(vbox, GAIMOSD_PREFS "/conv/toolbar",
	                 _("Show icon in toolbar"), 0);
#endif
	osd_config_check(vbox, GAIMOSD_PREFS "/conv/reply_hide",
	                 _("Hide OSD on reply"), 0);

	vbox = gaim_gtk_make_frame(page, _("Chat Conversations"));
#if BUILD_OSD_CONVS
	osd_config_check(vbox, GAIMOSD_PREFS "/chat/display_def",
	                 _("Default with OSD Display On"), 0);
	osd_config_check(vbox, GAIMOSD_PREFS "/chat/me_override",
	                 _("Always show messages with my screen name"), 0);
	osd_config_check(vbox, GAIMOSD_PREFS "/chat/toolbar",
	                 _("Show icon in toolbar"), 0);
#endif
	osd_config_check(vbox, GAIMOSD_PREFS "/chat/reply_hide",
	                 _("Hide OSD on reply"), 0);

	return page;
}

static GtkWidget *
osd_config_page_events(void)
{
	GtkWidget *page;
	GtkWidget *vbox, *hbox, *opt_menu, *menu, *item;
	GtkSizeGroup *sg;
	gint i;

	page = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	vbox = gaim_gtk_make_frame(page, _("Event"));
	menu = gtk_menu_new ();
	for (i = 0; i < N_EVENTS; i++) {
		item = gtk_menu_item_new_with_label (Osd_event[i].desc);
		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
		g_signal_connect (G_OBJECT (item), "activate",
		                  G_CALLBACK (osd_config_select_event),
		                  GINT_TO_POINTER (i));
	}
	opt_menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_menu), menu);
	gtk_box_pack_start (GTK_BOX (vbox), opt_menu, FALSE, FALSE, 0);
	gtk_widget_show(opt_menu);

	vbox = gaim_gtk_make_frame(page, _("Display Text"));
	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	Event_show_here_check = osd_config_check(vbox, GAIMOSD_PREFS "/event/@EVENT@/flags",
	                                         _("Show messages when here"),
	                                         GAIMOSD_FLAGS_SHOW_HERE);
	Event_show_away_check = osd_config_check(vbox, GAIMOSD_PREFS "/event/@EVENT@/flags",
	                                         _("Show messages when away"),
	                                         GAIMOSD_FLAGS_SHOW_AWAY);

	Event_buffer = osd_config_buffer(vbox, GAIMOSD_PREFS "/event/@EVENT@/text",
	                                 &Event_text_view);
	hbox = gtk_hbox_new (FALSE, 20);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	Event_color_button = osd_config_button(hbox, sg, gtk_button_new(),
	                                       _("Color"), FALSE,
	                                       G_CALLBACK(osd_color_dialog),
	                                       NULL);
	gtk_widget_set_size_request (Event_color_button, 100, -1);
	Event_color_label = gtk_label_new (NULL);
	gtk_container_add (GTK_CONTAINER (Event_color_button), Event_color_label);

	Event_delay_spin = osd_config_spin(hbox, sg, GAIMOSD_PREFS "/event/@EVENT@/timeout",
	                                   _("Duration"), 1, 60000,
	                                   100, 500, 500);
	Event_delay_spin_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (Event_delay_spin));

	Event_priority_spin = osd_config_spin(hbox, sg, GAIMOSD_PREFS "/event/@EVENT@/priority",
	                                      _("Priority"), 0, 100, 1, 10, 10);
	Event_priority_spin_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (Event_priority_spin));

	vbox = gaim_gtk_make_frame(page, _("Macros"));
	hbox = gtk_hbox_new (FALSE, 30);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	sg = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);

	vbox = gtk_vbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
	osd_config_macro (vbox, sg, "%u", _("= buddy's id"));
	osd_config_macro (vbox, sg, "%a", _("= buddy's alias"));
	osd_config_macro (vbox, sg, "%p", _("= buddy's protocol"));
	osd_config_macro (vbox, sg, "%m", _("= message"));

	vbox = gtk_vbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
	osd_config_macro (vbox, sg, "%t", _("= time, 12 hour format"));
	osd_config_macro (vbox, sg, "%T", _("= time, 24 hour format"));
	osd_config_macro (vbox, sg, "%d", _("= date (mm/dd/yy)"));
	osd_config_macro (vbox, sg, "%o", _("= chat room / warning level"));

	osd_config_select_event(NULL, NULL);

	return page;
}

#if BUILD_OSD_BUDDIES
static GtkWidget *
osd_config_page_buddies(void)
{
	GtkWidget *page = NULL;
	GtkWidget *vbox = NULL, *hbox = NULL;
	GtkWidget *button = NULL, *entry = NULL, *opt_menu = NULL;
	GtkSizeGroup *sg = NULL;
	GList *list = NULL;
	gint i;

	Buddies_selected = NULL;

	page = gtk_vbox_new (FALSE, 18);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	vbox = gaim_gtk_make_frame(page, _("Buddy"));

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

	Buddies_opt_menu = opt_menu = gtk_option_menu_new ();
	gtk_box_pack_start (GTK_BOX (hbox), opt_menu, TRUE, TRUE, 0);

	opt_menu = gaim_gtk_account_option_menu_new(NULL, TRUE,
	                                            G_CALLBACK(osd_buddies_select_account),
	                                            NULL, NULL);
	gtk_box_pack_start (GTK_BOX (hbox), opt_menu, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, 8);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	Buddies_entry = entry = gtk_entry_new ();
	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
	g_signal_connect (G_OBJECT (entry), "changed",
	                  G_CALLBACK (osd_buddies_search), entry);

	Buddies_remove = button = gtk_button_new_with_label (_("Remove"));
	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT (button), "clicked",
	                  G_CALLBACK (osd_buddies_remove), NULL);

	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	Buddies_priority_spin = osd_config_spin(vbox, sg, "priority",
	                                        _("Relative Priority"),
	                                        -100, 100, 1, 10, 10);
	Buddies_priority_spin_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (Buddies_priority_spin));

	vbox = gaim_gtk_make_frame(page, _("Events"));

	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	for (i = 0; i < N_EVENTS; i++) {
		osd_config_radio(vbox, sg, Osd_event[i].name,
	                         Osd_event[i].name, 3,
	                         _("Default"), GAIMOSD_FLAGS_SHOW_DEFAULT,
	                         &Buddies_radios[i*3+0],
	                         _("Always"), GAIMOSD_FLAGS_SHOW_ALWAYS,
	                         &Buddies_radios[i*3+1],
	                         _("Never"), GAIMOSD_FLAGS_SHOW_NEVER,
	                         &Buddies_radios[i*3+2]);
	}

	list = gaim_accounts_get_all();
	Buddies_account = (list) ? list->data : NULL;
	osd_buddies_select_account(NULL, Buddies_account);
	return page;
}
#endif /* BUILD_OSD_BUDDIES */

static GtkWidget *
osd_config_page_about(void)
{
	GtkWidget *page;
	GtkWidget *button, *scrolled, *text;
	GtkTextBuffer *buffer;
	GtkTextIter iter;
	GtkTextTag *bold_tag, *indent_tag;

	page = gtk_vbox_new (FALSE, 4);
	gtk_container_set_border_width (GTK_CONTAINER (page), 12);

	scrolled = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
	                                GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
	                                     GTK_SHADOW_IN);
	gtk_box_pack_start (GTK_BOX (page), scrolled, TRUE, TRUE, 0);

	text = gtk_text_view_new ();
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(text));
	gtk_text_buffer_get_start_iter (buffer, &iter);

	bold_tag = gtk_text_buffer_create_tag (buffer, "bold", "weight", "bold", NULL);
	indent_tag = gtk_text_buffer_create_tag (buffer, "indent", "left-margin", 20, NULL);

	gtk_text_buffer_insert_with_tags (buffer, &iter, _("Authors"), -1, bold_tag, NULL);
	gtk_text_buffer_insert_with_tags (buffer, &iter, "\n"
		"Paul A Miller <pmiller9@users.sourceforge.net>\n"
		"Chris Sperandio <csperandio@users.sourceforge.net>\n\n",
		-1, indent_tag, NULL);

	gtk_text_buffer_insert_with_tags (buffer, &iter, _("Patch Contributions"), -1, bold_tag, NULL);
	gtk_text_buffer_insert_with_tags (buffer, &iter, "\n"
		"Andrew J Cosgriff <ajc@users.sourceforge.net>\n"
		"Etan Reisner <deryni9@users.sourceforge.net>\n\n",
		-1, indent_tag, NULL);

	gtk_text_buffer_insert_with_tags (buffer, &iter, _("Bugs, Comments, Sugguestions, and Patches"), -1, bold_tag, NULL);
	gtk_text_buffer_insert_with_tags (buffer, &iter, _("\nPlease feel free to contact GaimOSDCreator (AIM) or visit our web page."), -1, indent_tag, NULL);

	gtk_text_buffer_get_start_iter (buffer, &iter);
	gtk_text_buffer_place_cursor (buffer, &iter);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(text), GTK_WRAP_WORD);
	gtk_text_view_set_editable (GTK_TEXT_VIEW(text), FALSE);
	gtk_container_add (GTK_CONTAINER (scrolled), text);

	button = gtk_button_new_with_label (GAIMOSD_WEBSITE);
	gtk_button_set_relief( GTK_BUTTON (button), GTK_RELIEF_NONE);
	gtk_box_pack_start (GTK_BOX (page), button, FALSE, FALSE, 0);
	g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK(osd_open_url), GAIMOSD_WEBSITE);

	return page;
}

static void
osd_config_close(GtkWidget *w, gpointer p)
{
	Dialog_open = 0;
}

static GtkWidget *
osd_config_dialog(GaimPlugin *plugin)
{
	GtkWidget *notebook;

	notebook = gtk_notebook_new ();
	Dialog_open = g_signal_connect(G_OBJECT(notebook), "destroy", G_CALLBACK(osd_config_close), NULL);

	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);

	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
	                          osd_config_page_display(),
	                          gtk_label_new (_("Display")));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
	                          osd_config_page_windows(),
	                          gtk_label_new (_("Windows")));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
	                          osd_config_page_events(),
	                          gtk_label_new (_("Events")));
#if BUILD_OSD_BUDDIES
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
	                          osd_config_page_buddies(),
	                          gtk_label_new (_("Custom Buddies")));
#endif
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
	                          osd_config_page_about(),
	                          gtk_label_new (_("About")));

	gtk_widget_show_all (notebook);

	return notebook;
}

static gboolean
plugin_load(GaimPlugin *plugin)
{
	Plugin = plugin;

#if BUILD_OSD_BUDDIES
	/* XXX Need to check to make sure this actually works the way I want it to
	 * and doesn't error when loaded at gaim start up */
	/*
	gaim_signal_connect(gaim_blist_get_handle(), "blist-node-extended-menu",
	                    plugin, GAIM_CALLBACK(osd_draw_buddy_menu), NULL);
	 */
#endif

	return TRUE;
}

static gboolean
plugin_unload(GaimPlugin *plugin)
{
	GaimPlugin *gaimosd = gaim_plugins_find_with_id("gaimosd");

	gaim_plugin_unload(gaimosd);

	return TRUE;
}

static GaimGtkPluginUiInfo ui_info =
{
	osd_config_dialog
};

static GaimPluginInfo info =
{
	GAIM_PLUGIN_MAGIC,
	GAIM_MAJOR_VERSION,
	GAIM_MINOR_VERSION,
	GAIM_PLUGIN_STANDARD,                            /**< type           */
	GAIM_GTK_PLUGIN_TYPE,                            /**< ui_requirement */
	0,                                               /**< flags          */
	NULL,                                            /**< dependencies   */
	GAIM_PRIORITY_DEFAULT,                           /**< priority       */

	GAIMOSD_GTK_PLUGIN_ID,                           /**< id             */
	N_("Gaim OSD"),                                  /**< name           */
	GAIMOSD_VERSION,                                 /**< version        */
	                                                 /**< summary        */
	N_("On-screen display messages."),
	                                                 /**< description    */
	N_("Adds on-screen notification messages.  The effect is "
	  "similar to text overlay on a television for menus, volume, and "
	  "contrast control."),
	                                                 /**< author         */
	"\n\tPaul Miller <pmiller9@users.sourceforge.net>\n"
	"\tChris Sperandio <csperandio@users.sourceforge.net>\n",
	GAIMOSD_WEBSITE,                                 /**< homepage       */

	plugin_load,                                     /**< load           */
	plugin_unload,                                   /**< unload         */
	NULL,                                            /**< destroy        */

	&ui_info,                                        /**< ui_info        */
	NULL                                             /**< extra_info     */
};

static void
init_plugin(GaimPlugin *plugin)
{
	info.dependencies = g_list_append(info.dependencies, "gaimosd");
}

GAIM_INIT_PLUGIN(gaimosdgtk, init_plugin, info);
