/*
 * gaim
 *
 * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
 *
 * osd.c: On-screen display plugin for Gaim.
 * Copyright (C) 2001 Chris Sperandio <csperandio@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/* (cvs $Id: hooks.c,v 1.16 2004/07/21 19:59:10 deryni9 Exp $) */

#include "gaimosd.h"

/*void
osd_display(GaimConnection *gc, GaimConversation *c, gchar *who, gint event, gchar *m, gchar *o)
*/

static GaimConversation *
osd_find_chat(gint id)
{
	GaimConversation *conv;
	GList *chats;
	gint i;

	chats = gaim_get_chats();
	while (chats) {
		conv = (GaimConversation *) chats->data;
		i = gaim_conv_chat_get_id(gaim_conversation_get_chat_data(conv));
		if (i == id) return conv;
		chats = chats->next;
	}
	return NULL;
}

/**************************************************************************
 * Account subsystem signal callbacks
 **************************************************************************/
static void
osd_account_warned_cb(GaimAccount *account, gchar *warner, int level, void *data)
{
	GaimConnection *gc = gaim_account_get_connection(account);
	GaimConversation *conv = gaim_find_conversation_with_account(warner, account);
	gchar *o = g_strdup_printf("%d", level);

	osd_display(gc,conv,warner,EVENT_ACCOUNT_WARNED,NULL,o);
	g_free(o);
}

/**************************************************************************
 * Buddy List subsystem signal callbacks
 **************************************************************************/
static void
osd_buddy_subsystem_cb(GaimBuddy *buddy, void *data)
{
	GaimConnection *gc;
	GaimConversation *conv;

	if (!buddy) return;

	gc = gaim_account_get_connection(buddy->account);
	conv = gaim_find_conversation_with_account(buddy->name, buddy->account);
	osd_display(gc,conv,buddy->name,GPOINTER_TO_INT(data),NULL,NULL);
}

/**************************************************************************
 * Connection subsystem signal callbacks
 **************************************************************************/
static void
osd_signed_on_cb(GaimConnection *gc, void *data)
{
	osd_display_t *disp;

	disp = g_new0(osd_display_t,1);
	if (!disp)
		return;

	disp->priority = gaim_prefs_get_int(GAIMOSD_PREFS "/signon_priority");
	disp->lines = -Osd_lines;
	disp->slist = NULL;

	osd_display_kill();
	Osd_current = disp;
	disp->timeout = g_timeout_add(gaim_prefs_get_int(GAIMOSD_PREFS "/signon_timeout"), (GSourceFunc) osd_display_cb, disp);
}

/**************************************************************************
 * Conversation subsystem signal callbacks
 **************************************************************************/
static gboolean
osd_sending_im_msg_cb(GaimAccount *account, const gchar *recipient, gchar **buffer, void *data)
{
	GaimConversation *conv = gaim_find_conversation_with_account(recipient, account);

	if (Osd_current && Osd_current->c == conv
			&& gaim_prefs_get_bool(GAIMOSD_PREFS "/conv/reply_hide"))
		osd_display_kill();
	return FALSE;
}

static gboolean
osd_received_im_msg_cb(GaimAccount *account, gchar *sender, gchar *buffer, int *flags, void *data)
{
	GaimConnection *gc = gaim_account_get_connection(account);
	GaimConversation *conv;
	gchar *stripped;

	if (!*sender || !*buffer)
		return FALSE;

	conv = gaim_find_conversation_with_account(sender, account);
	stripped = gaim_markup_strip_html(buffer);

	osd_display(gc, conv, sender, EVENT_RECEIVED_IM_MSG, stripped, NULL);
	g_free(stripped);
	return FALSE;
}

static gboolean
osd_sending_chat_msg_cb(GaimAccount *account, gchar **buffer, int id, void *data)
{
	if (Osd_current && (Osd_current->c == osd_find_chat(id))
			&& gaim_prefs_get_bool(GAIMOSD_PREFS "/chat/reply_hide"))
		osd_display_kill();
	return FALSE;
}

static gboolean
osd_received_chat_msg_cb(GaimAccount *account, gchar *sender, gchar *buffer, GaimConversation *conv, void *data)
{
	GaimConnection *gc;
	gchar *stripped, *usertext, *username, *room, *new, *pos;

	if (!(*buffer))
		return FALSE;

	gc = gaim_account_get_connection(account);
	stripped = gaim_markup_strip_html(buffer);
	/* XXX I can't imagine this is UTF-8 safe.
	 * Do we need to be? */
	usertext = g_ascii_strdown(stripped, -1);
	username = g_ascii_strdown(gaim_account_get_username(account), -1);
	room = (gchar *)gaim_conversation_get_name(conv);

/* XXX Do I need to replace this with anything? */
#if 0
	if ((gaim_account_get_protocol(account) == GAIM_PROTO_IRC) && (strncmp(usertext, "/me ", 4) == 0)) {
		new = g_strconcat(sender, stripped+3, NULL);
		g_free(stripped);
		stripped = new;
	}
#endif
	if ((pos = strchr(username, '@'))) {
		new = g_strndup(username, pos - username);
		g_free(username);
		username = new;
	}

	/* check if my username was mentioned */
	pos = strstr(usertext, username);
	if (pos && ((pos == usertext || !isalnum(pos[-1])) && (!isalnum(pos[strlen(username)])))) {
		osd_display(gc, conv, sender, EVENT_RECEIVED_CHAT_MSG_ME, stripped, room);
	} else {
		osd_display(gc, conv, sender, EVENT_RECEIVED_CHAT_MSG, stripped, room);
	}

	g_free(stripped);
	g_free(usertext);
	g_free(username);
	return FALSE;
}

static void
osd_conversation_created_cb(GaimConversation *conv, void *data)
{
	GaimConnection *gc;
	gchar *who;

#if BUILD_OSD_CONVS
	osd_conv_new(conv);
#endif
	if (gaim_conversation_get_type(conv) == GAIM_CONV_CHAT) return;

	gc = gaim_conversation_get_gc(conv);
	who = (gchar *)gaim_conversation_get_name(conv);

	osd_display(gc,conv,who,EVENT_CONVERSATION_CREATED,NULL,NULL);
}

static void
osd_deleting_conversation_cb(GaimConversation *conv, void *data)
{
	if (Osd_current && Osd_current->c == conv) osd_display_kill();
#if BUILD_OSD_CONVS
	osd_conv_delete(conv);
#endif
}

static void
osd_buddy_typing_cb(GaimConversation *conv, void *data)
{
	GaimConnection *gc = gaim_conversation_get_gc(conv);
	gchar *who = (gchar *)gaim_conversation_get_name(conv);

	osd_display(gc,conv,who,EVENT_BUDDY_TYPING,NULL,NULL);
}

static void
osd_chat_buddy_joined_cb(GaimConversation *conv, const gchar *user, void *data)
{
	GaimConnection *gc = gaim_conversation_get_gc(conv);
	gchar *room = (gchar *)gaim_conversation_get_name(conv);

	osd_display(gc, conv, (char *)user, EVENT_CHAT_BUDDY_JOINED, NULL, room);
}

static void
osd_chat_buddy_left_cb(GaimConversation *conv, const gchar *user, const gchar *reason, void *data)
{
	GaimConnection *gc = gaim_conversation_get_gc(conv);
	gchar *room = (gchar *)gaim_conversation_get_name(conv);

	osd_display(gc, conv, (char *)user, EVENT_CHAT_BUDDY_LEFT, (char *)reason, room);
}

static void
osd_chat_invited_cb(GaimAccount *account, const gchar *inviter, const gchar *invite, const gchar *message, void *data)
{
	GaimConnection *gc = gaim_account_get_connection(account);

	osd_display(gc, NULL, (char *)inviter, EVENT_CHAT_INVITED, (char *)message, (char *)invite);
}

static void
osd_chat_joined_cb(GaimConversation *conv, void *data)
{
	GaimConnection *gc = gaim_conversation_get_gc(conv);
	gchar *room = (gchar *)gaim_conversation_get_name(conv);
	gchar *topic = (gchar *)gaim_conv_chat_get_topic(gaim_conversation_get_chat_data(conv));

	osd_display(gc,conv,NULL,EVENT_CHAT_JOINED,topic,room);
}

void
osd_hooks_connect(GaimPlugin *plugin)
{
	void *blist_handle = gaim_blist_get_handle();
	void *conn_handle = gaim_connections_get_handle();
	void *conv_handle = gaim_conversations_get_handle();
	void *accounts_handle = gaim_accounts_get_handle();

	/* Accounts subsystem signals */
	gaim_signal_connect(accounts_handle, "account-warned",
		                  plugin, GAIM_CALLBACK(osd_account_warned_cb), NULL);

	/* Buddy List subsystem signals */
	gaim_signal_connect(blist_handle, "buddy-away",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_AWAY));
	gaim_signal_connect(blist_handle, "buddy-back",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_BACK));
	gaim_signal_connect(blist_handle, "buddy-idle",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_IDLE));
	gaim_signal_connect(blist_handle, "buddy-unidle",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_UNIDLE));
	gaim_signal_connect(blist_handle, "buddy-signed-on",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_SIGNED_ON));
	gaim_signal_connect(blist_handle, "buddy-signed-off",
		                  plugin, GAIM_CALLBACK(osd_buddy_subsystem_cb),
		                  GINT_TO_POINTER(EVENT_BUDDY_SIGNED_OFF));

	/* Connection subsystem signals */
	gaim_signal_connect(conn_handle, "signed-on",
		                  plugin, GAIM_CALLBACK(osd_signed_on_cb), NULL);

	/* Conversations subsystem signals */
	gaim_signal_connect(conv_handle, "sending-im-msg",
		                  plugin, GAIM_CALLBACK(osd_sending_im_msg_cb), NULL);
	gaim_signal_connect(conv_handle, "received-im-msg",
		                  plugin, GAIM_CALLBACK(osd_received_im_msg_cb), NULL);
	gaim_signal_connect(conv_handle, "sending-chat-msg",
		                  plugin, GAIM_CALLBACK(osd_sending_chat_msg_cb), NULL);
	gaim_signal_connect(conv_handle, "received-chat-msg",
		                  plugin, GAIM_CALLBACK(osd_received_chat_msg_cb), NULL);
	gaim_signal_connect(conv_handle, "conversation-created",
		                  plugin, GAIM_CALLBACK(osd_conversation_created_cb), NULL);
	gaim_signal_connect(conv_handle, "deleting-conversation",
		                  plugin, GAIM_CALLBACK(osd_deleting_conversation_cb), NULL);
	gaim_signal_connect(conv_handle, "buddy-typing",
		                  plugin, GAIM_CALLBACK(osd_buddy_typing_cb), NULL);
	gaim_signal_connect(conv_handle, "chat-buddy-joined",
		                  plugin, GAIM_CALLBACK(osd_chat_buddy_joined_cb), NULL);
	gaim_signal_connect(conv_handle, "chat-buddy-left",
		                  plugin, GAIM_CALLBACK(osd_chat_buddy_left_cb), NULL);
	gaim_signal_connect(conv_handle, "chat-invited",
		                  plugin, GAIM_CALLBACK(osd_chat_invited_cb), NULL);
	gaim_signal_connect(conv_handle, "chat-joined",
		                  plugin, GAIM_CALLBACK(osd_chat_joined_cb), NULL);
}
